# Revision 1.1 2007/04/21 19:50:00 iisaka
# initial release

import os, sys
import string
import getpass

class ScatterRun:
    def __init__(self, execcmd, user=None, jobmode="seq" ):
        import tempfile
        self.scriptbuf=[]
        self.tmpdir= tempfile.mktemp("ScatterCmd")
        self.script = self.tmpdir + "/ScatterRun.sh"
        self.waitcmd = ''
        self.bgjob = ''
        self.comment = ''
        self.method = execcmd
        self.printmode = "N"
        self.verbose = "N"
        self.color = "C"
        if user == None:
            self.user = getpass.getuser()
        else:
	    self.user = user

        try:
           os.mkdir( self.tmpdir )
        except  IOError, emsg:
           sys.stderr.write( "%s\n" % emsg )
           sys.exit(1)

        if jobmode == None:
            self.jobmode = "seq"
        else:
            self.jobmode = string.lower(jobmode)

        if self.jobmode == "broad" or  self.jobmode == "broadcast":
           self.jobmode = "B"
           self.waitcmd = "wait"
           self.bgjob = "&"
           self.printmode = "N"
        elif self.jobmode == "seq" or self.jobmode == "sequence":
           self.jobmode = "S"
        elif self.jobmode == "intract" or self.jobmode == "intractive":
           self.jobmode = "I"
           self.printmode = "D"
        elif self.jobmode == "eval":
           self.jobmode = "D"
        else:
           raise ValueError, "Invalid type: %s" % ( self.jobmode )

        self.shHeader="""#__SUPPRESS__
SAVEDIR=`pwd`                   
export ScatterTMPDIR=%s
cd ${ScatterTMPDIR}
[ -f  /usr/local/libexec/ScatterCmd/libScatterRun ] && { 
    .  /usr/local/libexec/ScatterCmd/libScatterRun
}                                                    
""" % self.tmpdir

        self.shFooter="""#__SUPPRESS__
cd $SAVEDIR
SCRemoveFifo 
"""

    def __del__(self):
	pass

    def close(self):
        os.system( "/bin/rm -rf " + self.tmpdir )
	pass

    def setComment(self, comment):
	if comment != None:
	    self.comment = comment

    def buildHeader(self):
        self.scriptbuf.append( self.shHeader )

    def buildFooter(self):
        self.scriptbuf.append( self.shFooter)

    def makeFifo(self, node):
        self.scriptbuf.append("SCMakeFifo %s #__SUPPRESS__\n" % ( node ))

    def setFifo(self, node):
        if self.printmode != "Q":
            self.scriptbuf.append("SCSetFifo %s #__SUPPRESS__\n" % ( node ))

    def buildCmdLine( self, node, args ):
	args = string.replace( args, '@H@', node )
	args = string.replace( args, '@C@', os.getcwd() )
        if ( self.jobmode == 'D' ):
            self.scriptbuf.append( 'eval "%s" 1>po.%s 2>pe.%s %s\n' % \
                (args, node, node, self.bgjob))
        elif ( self.printmode == 'Q' ):
            self.scriptbuf.append( '%s -l %s %s "%s" 1>o.%s 2>e.%s %s\n' % \
                (self.method, self.user, node, args, node, node, self.bgjob))
        else:
            self.scriptbuf.append( '%s -l %s %s "%s" 1>po.%s 2>pe.%s %s\n' % \
                (self.method, self.user, node, args, node, node, self.bgjob))

    def buildComment( self, str ):
            self.scriptbuf.append( '# %s \n' % str )

    def buildShowCmd( self, node, comment ):
        if node == "" and self.waitcmd != "":
            self.scriptbuf.append( self.waitcmd )
            self.scriptbuf.append( "\n" )
        elif node != "":
            self.scriptbuf.append("SCPrintNodeName %s %s %s #__SUPPRESS__\n" % \
                                  (self.printmode, self.color, node ))
            self.scriptbuf.append("SCPrintResult %s %s %s   #__SUPPRESS__\n" % \
                                  (self.printmode, node, comment ) )

    def buildScript(self, nodeList, cmdline):
	self.buildHeader()
	if self.isBGJob():
            for n in nodeList:
                self.makeFifo(n)
                self.setFifo(n)
	        self.buildCmdLine( n, cmdline)
	    for n in nodeList:
                self.buildShowCmd(n, self.comment)
	else:
            for n in nodeList:
                self.makeFifo(n)
                self.setFifo(n)
	        self.buildCmdLine(n, cmdline)
                self.buildShowCmd(n, self.comment)
	self.buildFooter()

    def initScript(self):
        self.scriptbuf=[]

    def setVerbose(self, verbose):
        if verbose:
            self.verbose = "V"
        else:
            self.verbose = "N"

    def setColorMode(self, do_monochrome):
        if do_monochrome:
            self.color = "M"
        else:
            self.color = "C"

    def setPrintmode(self, silent, quiet):
        if quiet :
            self.printmode = "Q"
        elif silent:
            self.printmode = "S"

    def isBGJob(self):
        if self.jobmode == "B":
            return True
        else:
            return False

    def getWorkDir(self):
            return self.tmpdir

    def writeOut(self):
            try:
                fd = open( self.script, "w")
            except IOError :
                sys.stderr.write( "Could not write workfile." )
                self.close()
                sys.exit(1)
            else:
                fd.writelines( self.scriptbuf )
                fd.close()

    def showScript(self):
            for L in self.scriptbuf:
                try:
                   # WARRNING!
                   # This is a durty & cheat trick. ;-)
                   # Triple-quoted strings are sepalated by '\012'.
                   string.index(L, "#__SUPPRESS__")
                except:
                    sys.stdout.write( L )
                else:
                    pass

    def doScript(self, outputfile='' ):
        if outputfile != '' and outputfile != None:
            output = ' > ' + outputfile
        else:
            output = ''
        if self.script:
            cmdline = "sh " + self.script + output 
            os.system(cmdline)

    def copyFiles(self, distdir=''):
        if distdir == "":
           distdir = os.environ("HOME")
        os.system( "cp %s/* %s" % (self.tmpdir, distdir ))

